@@ -1,4 +1,6 @@
 import base64
+import copy
+import datetime
 import hashlib
 import logging
 import os
@@ -12,8 +14,6 @@ from http.client import HTTPConnection
 from json import dumps
 from typing import Any, Dict, List, Optional, Tuple
 from urllib.parse import urljoin
-import datetime
-import copy
 
 import jwt
 import pandas as pd
@@ -22,6 +22,7 @@ from pydantic import BaseModel
 from pythonjsonlogger import jsonlogger
 from requests.exceptions import RequestException
 
+from upgini.__about__ import __version__
 from upgini.errors import (
     HttpError,
     UnauthorizedError,
@@ -38,17 +39,17 @@ from upgini.metadata import (
 from upgini.resource_bundle import bundle
 from upgini.utils.track_info import get_track_metrics
 
-try:
-    from importlib_metadata import version  # type: ignore
+# try:
+#     from importlib.metadata import version  # type: ignore
 
-    __version__ = version("upgini")
-except ImportError:
-    try:
-        from importlib.metadata import version  # type: ignore
+#     __version__ = version("upgini")
+# except ImportError:
+#     try:
+#         from importlib_metadata import version  # type: ignore
 
-        __version__ = version("upgini")
-    except ImportError:
-        __version__ = "Upgini wasn't installed"
+#         __version__ = version("upgini")
+#     except ImportError:
+#         __version__ = "Upgini wasn't installed"
 
 UPGINI_URL: str = "UPGINI_URL"
 UPGINI_API_KEY: str = "UPGINI_API_KEY"
@@ -185,7 +186,11 @@ class SearchProgress:
         self.eta = int(seconds_left * (100 - self.percent) / self.percent)
 
     def update_eta(self, seconds: int):
-        self.eta = seconds
+        # temporary workaround - add 5 minutes if time is over
+        if seconds <= 0:
+            self.eta = 5 * 60
+        else:
+            self.eta = seconds
 
     def eta_time(self) -> str:
         return str(datetime.timedelta(seconds=self.eta))
@@ -207,6 +212,7 @@ class ProgressStage(Enum):
     CREATING_FIT = "CREATING_FIT"
     CREATING_TRANSFORM = "CREATING_TRANSFORM"
     MATCHING = "MATCHING"
+    AUTOFE = "AUTOFE"
     SEARCHING = "SEARCHING"
     ENRICHING = "ENRICHING"
     GENERATING_REPORT = "GENERATING_REPORT"
@@ -224,6 +230,21 @@ class LogEvent(BaseModel):
     message: str
 
 
+class TransformUsage:
+    def __init__(self, json: dict):
+        self.transformed_rows = json.get("transformedRows")
+        self.rest_rows = json.get("restRows")
+        self.limit = json.get("limit")
+        self.has_limit = json.get("hasLimit")
+
+    def __str__(self):
+        return (
+            f"TransformUsage(transformed_rows={self.transformed_rows}, "
+            f"rest_rows={self.rest_rows}, limit={self.limit}, "
+            f"has_limit={self.has_limit})"
+        )
+
+
 class _RestClient:
     PROD_BACKEND_URL = "https://search.upgini.com"
 
@@ -254,6 +275,7 @@ class _RestClient:
     SEARCH_FILE_METADATA_URI_FMT_V2 = SERVICE_ROOT_V2 + "search/{0}/metadata"
     SEARCH_TASK_METADATA_FMT_V3 = SERVICE_ROOT_V2 + "search/metadata-v2/{0}"
     SEARCH_DUMP_INPUT_FMT_V2 = SERVICE_ROOT_V2 + "search/dump-input"
+    TRANSFORM_USAGE_FMT = SERVICE_ROOT_V2 + "user/transform-usage"
 
     UPLOAD_USER_ADS_URI = SERVICE_ROOT + "ads/upload"
     SEND_LOG_EVENT_URI = "private/api/v2/events/send"
@@ -263,10 +285,14 @@ class _RestClient:
     DEACTIVATE_ADS_URI = "private/api/v2/ads/deactivate"
     TOGGLE_ADS_URI_FMT = "private/api/v2/ads/{0}/toggle"
     DELETE_ADS_URI_FMT = "private/api/v2/ads/{0}"
+    REANNOUNCE_ALL_ADS_URI_FMT = "private/api/v2/ads/reannounce-all-ads"
     POLL_ADS_MANAGEMENT_STATUS_URI_FMT = "private/api/v2/ads/management-task/{0}"
     GET_ADS_DESCRIPTION_URI_FMT = "private/api/v2/ads/{0}"
     GET_ALL_ADS_DESCRIPTIONS_URI = "private/api/v2/ads/descriptions"
     GET_ACTIVE_ADS_DEFINITIONS_URI = "private/api/v2/ads/definitions"
+    UPLOAD_ONLINE_URI = "private/api/v2/ads/upload-online"
+    STOP_ADS_MANAGEMENT_TASK_URI_FMT = "private/api/v2/ads/management-task/{0}/stop"
+    UNION_SEARCH_TASKS_URI_FMT = SERVICE_ROOT_V2 + "search/merge"
 
     ACCESS_TOKEN_HEADER_NAME = "Authorization"
     CONTENT_TYPE_HEADER_NAME = "Content-Type"
@@ -278,11 +304,13 @@ class _RestClient:
     USER_AGENT_HEADER_VALUE = "pyupgini/" + __version__
     SEARCH_KEYS_HEADER_NAME = "Search-Keys"
 
-    def __init__(self, service_endpoint, refresh_token, silent_mode=False):
+    def __init__(self, service_endpoint, refresh_token, client_ip=None, client_visitorid=None):
         # debug_requests_on()
         self._service_endpoint = service_endpoint
         self._refresh_token = refresh_token
-        self.silent_mode = silent_mode
+        # self.silent_mode = silent_mode
+        self.client_ip = client_ip
+        self.client_visitorid = client_visitorid
         self._access_token = self._refresh_access_token()
         # self._access_token: Optional[str] = None  # self._refresh_access_token()
         self.last_refresh_time = time.time()
@@ -335,6 +363,7 @@ class _RestClient:
                 time.sleep(10)
                 return self._with_unauth_retry(request)
             else:
+                print(e)
                 show_status_error()
                 raise e
         except UnauthorizedError:
@@ -353,6 +382,7 @@ class _RestClient:
             elif "more than one concurrent search request" in e.message.lower():
                 raise ValidationError(bundle.get("concurrent_request"))
             else:
+                print(e)
                 show_status_error()
                 raise e
 
@@ -414,6 +444,8 @@ class _RestClient:
     ) -> SearchTaskResponse:
         api_path = self.INITIAL_SEARCH_URI_FMT_V2
 
+        track_metrics = get_track_metrics(self.client_ip, self.client_visitorid)
+
         def open_and_send():
             md5_hash = hashlib.md5()
             with open(file_path, "rb") as file:
@@ -434,6 +466,11 @@ class _RestClient:
                         metadata_with_md5.json(exclude_none=True).encode(),
                         "application/json",
                     ),
+                    "tracking": (
+                        "tracking.json",
+                        dumps(track_metrics).encode(),
+                        "application/json",
+                    ),
                     "metrics": ("metrics.json", metrics.json(exclude_none=True).encode(), "application/json"),
                     "file": (metadata_with_md5.name, file, "application/octet-stream"),
                 }
@@ -443,18 +480,13 @@ class _RestClient:
                         search_customization.json(exclude_none=True).encode(),
                         "application/json",
                     )
-                files["tracking"] = (
-                    "tracking.json",
-                    dumps(get_track_metrics()).encode(),
-                    "application/json",
-                )
                 additional_headers = {self.SEARCH_KEYS_HEADER_NAME: ",".join(self.search_keys_meaning_types(metadata))}
 
                 return self._send_post_file_req_v2(
                     api_path, files, trace_id=trace_id, additional_headers=additional_headers
                 )
 
-        response = self._with_unauth_retry(lambda: open_and_send())
+        response = self._with_unauth_retry(open_and_send)
         return SearchTaskResponse(response)
 
     def check_uploaded_file_v2(self, trace_id: str, file_upload_id: str, metadata: FileMetadata) -> bool:
@@ -518,6 +550,11 @@ class _RestClient:
                         metadata_with_md5.json(exclude_none=True).encode(),
                         "application/json",
                     ),
+                    "tracking": (
+                        "tracking.json",
+                        dumps(get_track_metrics(self.client_ip, self.client_visitorid)).encode(),
+                        "application/json",
+                    ),
                     "metrics": ("metrics.json", metrics.json(exclude_none=True).encode(), "application/json"),
                     "file": (metadata_with_md5.name, file, "application/octet-stream"),
                 }
@@ -527,11 +564,6 @@ class _RestClient:
                         search_customization.json(exclude_none=True).encode(),
                         "application/json",
                     )
-                files["tracking"] = (
-                    "ide",
-                    dumps(get_track_metrics()).encode(),
-                    "application/json",
-                )
 
                 additional_headers = {self.SEARCH_KEYS_HEADER_NAME: ",".join(self.search_keys_meaning_types(metadata))}
 
@@ -539,7 +571,7 @@ class _RestClient:
                     api_path, files, trace_id=trace_id, additional_headers=additional_headers
                 )
 
-        response = self._with_unauth_retry(lambda: open_and_send())
+        response = self._with_unauth_retry(open_and_send)
         return SearchTaskResponse(response)
 
     def validation_search_without_upload_v2(
@@ -636,6 +668,16 @@ class _RestClient:
         response = self._with_unauth_retry(lambda: self._send_get_req(api_path, trace_id))
         return ProviderTaskMetadataV2.parse_obj(response)
 
+    def get_current_transform_usage(self, trace_id) -> TransformUsage:
+        track_metrics = get_track_metrics(self.client_ip, self.client_visitorid)
+        visitor_id = track_metrics.get("visitorId")
+        response = self._with_unauth_retry(
+            lambda: self._send_get_req(
+                self.TRANSFORM_USAGE_FMT, trace_id, additional_headers={"Visitor-Id": visitor_id}
+            )
+        )
+        return TransformUsage(response)
+
     def send_log_event(self, log_event: LogEvent):
         api_path = self.SEND_LOG_EVENT_URI
         try:
@@ -685,6 +727,11 @@ class _RestClient:
         api_path = self.DEACTIVATE_ADS_URI
         self._with_unauth_retry(lambda: self._send_post_req(api_path, trace_id, request, result_format=None))
 
+    def reannounce_all_ads(self, trace_id: str) -> str:
+        api_path = self.REANNOUNCE_ALL_ADS_URI_FMT
+        response = self._with_unauth_retry(lambda: self._send_post_req(api_path, trace_id))
+        return response["adsManagementTaskId"]
+
     def toggle_ads(self, ads_definition_id: str, trace_id: str):
         api_path = self.TOGGLE_ADS_URI_FMT.format(ads_definition_id)
         return self._with_unauth_retry(lambda: self._send_post_req(api_path, trace_id))
@@ -711,11 +758,25 @@ class _RestClient:
         except Exception:
             return None
 
+    def upload_online(self, request: dict, trace_id: str):
+        api_path = self.UPLOAD_ONLINE_URI
+        response = self._with_unauth_retry(lambda: self._send_post_req(api_path, trace_id, request))
+        return response["adsManagementTaskId"]
+
+    def stop_ads_management_task(self, ads_management_task_id: str, trace_id: str):
+        api_path = self.STOP_ADS_MANAGEMENT_TASK_URI_FMT.format(ads_management_task_id)
+        self._with_unauth_retry(lambda: self._send_post_req(api_path, trace_id))
+
+    def union_search_tasks(self, request: dict, trace_id: str):
+        api_path = self.UNION_SEARCH_TASKS_URI_FMT
+        return self._with_unauth_retry(lambda: self._send_post_req(api_path, trace_id, request, result_format=None))
+
     # ---
 
-    def _send_get_req(self, api_path: str, trace_id: Optional[str]):
+    def _send_get_req(self, api_path: str, trace_id: Optional[str], additional_headers: Optional[dict] = None):
         response = requests.get(
-            url=urljoin(self._service_endpoint, api_path), headers=self._get_headers(trace_id=trace_id)
+            url=urljoin(self._service_endpoint, api_path),
+            headers=self._get_headers(trace_id=trace_id, additional_headers=additional_headers),
         )
 
         if response.status_code >= 400:
@@ -860,34 +921,50 @@ def resolve_api_token(api_token: Optional[str]) -> str:
         return DEMO_API_KEY
 
 
-def get_rest_client(backend_url: Optional[str] = None, api_token: Optional[str] = None) -> _RestClient:
+def get_rest_client(
+    backend_url: Optional[str] = None,
+    api_token: Optional[str] = None,
+    client_ip: Optional[str] = None,
+    client_visitorid: Optional[str] = None,
+) -> _RestClient:
     url = _resolve_backend_url(backend_url)
     token = resolve_api_token(api_token)
 
-    return _get_rest_client(url, token)
+    return _get_rest_client(url, token, client_ip, client_visitorid)
 
 
 def is_demo_api_key(api_token: Optional[str]) -> bool:
     return api_token is None or api_token == "" or api_token == DEMO_API_KEY
 
 
-@lru_cache()
-def _get_rest_client(backend_url: str, api_token: str) -> _RestClient:
-    return _RestClient(backend_url, api_token)
+@lru_cache
+def _get_rest_client(
+    backend_url: str, api_token: str, client_ip: Optional[str] = None, client_visitorid: Optional[str] = None
+) -> _RestClient:
+    return _RestClient(backend_url, api_token, client_ip, client_visitorid)
 
 
 class BackendLogHandler(logging.Handler):
-    def __init__(self, rest_client: _RestClient, *args, **kwargs) -> None:
+    def __init__(
+        self,
+        rest_client: _RestClient,
+        client_ip: Optional[str] = None,
+        client_visitorid: Optional[str] = None,
+        *args,
+        **kwargs,
+    ) -> None:
         super().__init__(*args, **kwargs)
         self.rest_client = rest_client
         self.track_metrics = None
         self.hostname = "0.0.0.0"
+        self.client_ip = client_ip
+        self.client_visitorid = client_visitorid
 
     def emit(self, record: logging.LogRecord) -> None:
         def task():
             try:
                 if self.track_metrics is None or len(self.track_metrics) == 0:
-                    self.track_metrics = get_track_metrics()
+                    self.track_metrics = get_track_metrics(self.client_ip, self.client_visitorid)
                     self.hostname = self.track_metrics.get("ip") or "0.0.0.0"
                 text = self.format(record)
                 tags = self.track_metrics
@@ -928,7 +1005,13 @@ class LoggerFactory:
         root.setLevel(logging.INFO)
         root.handlers.clear()
 
-    def get_logger(self, backend_url: Optional[str] = None, api_token: Optional[str] = None) -> logging.Logger:
+    def get_logger(
+        self,
+        backend_url: Optional[str] = None,
+        api_token: Optional[str] = None,
+        client_ip: Optional[str] = None,
+        client_visitorid: Optional[str] = None,
+    ) -> logging.Logger:
         url = _resolve_backend_url(backend_url)
         token = resolve_api_token(api_token)
         key = url + token
@@ -938,8 +1021,8 @@ class LoggerFactory:
 
         upgini_logger = logging.getLogger(f"upgini.{hash(key)}")
         upgini_logger.handlers.clear()
-        rest_client = get_rest_client(backend_url, api_token)
-        datadog_handler = BackendLogHandler(rest_client)
+        rest_client = get_rest_client(backend_url, api_token, client_ip, client_visitorid)
+        datadog_handler = BackendLogHandler(rest_client, client_ip, client_visitorid)
         json_formatter = jsonlogger.JsonFormatter(
             "%(asctime)s %(threadName)s %(name)s %(levelname)s %(message)s",
             timestamp=True,
