@@ -3,7 +3,8 @@ from logging import Logger
 from typing import List, Optional
 
 import pandas as pd
-from pandas.api.types import is_object_dtype, is_integer_dtype, is_string_dtype
+from pandas.api.types import is_integer_dtype, is_object_dtype, is_string_dtype
+
 from upgini.resource_bundle import bundle
 from upgini.utils.warning_counter import WarningCounter
 
@@ -16,9 +17,16 @@ class FeaturesValidator:
             self.logger = logging.getLogger()
             self.logger.setLevel("FATAL")
 
-    def validate(self, df: pd.DataFrame, features: List[str], warning_counter: WarningCounter) -> List[str]:
+    def validate(
+        self,
+        df: pd.DataFrame,
+        features: List[str],
+        features_for_generate: Optional[List[str]],
+        warning_counter: WarningCounter,
+    ) -> List[str]:
         # one_hot_encoded_features = []
         empty_or_constant_features = []
+        high_cardinality_features = []
 
         for f in features:
             column = df[f]
@@ -51,15 +59,32 @@ class FeaturesValidator:
             msg = bundle.get("empty_or_contant_features").format(empty_or_constant_features)
             print(msg)
             self.logger.warning(msg)
+            warning_counter.increment()
 
-        return empty_or_constant_features
+        high_cardinality_features = self.find_high_cardinality(df[features])
+        if features_for_generate:
+            high_cardinality_features = [f for f in high_cardinality_features if f not in features_for_generate]
+        if high_cardinality_features:
+            msg = bundle.get("high_cardinality_features").format(high_cardinality_features)
+            print(msg)
+            self.logger.warning(msg)
+            warning_counter.increment()
+
+        return empty_or_constant_features + high_cardinality_features
 
     @staticmethod
-    def find_high_cardinality(df: pd.DataFrame):
+    def find_high_cardinality(df: pd.DataFrame) -> List[str]:
         # Remove high cardinality columns
         row_count = df.shape[0]
+        if row_count < 100:  # For tests with small datasets
+            return []
         return [
             i
             for i in df
-            if (is_string_dtype(df[i]) or is_integer_dtype(df[i])) and (df[i].nunique() / row_count >= 0.9)
+            if (is_object_dtype(df[i]) or is_string_dtype(df[i]) or is_integer_dtype(df[i]))
+            and (df[i].nunique(dropna=False) / row_count >= 0.85)
         ]
+
+    @staticmethod
+    def find_constant_features(df: pd.DataFrame) -> List[str]:
+        return [i for i in df if df[i].nunique() == 1]
